class WebController < ApplicationController

	protected

	def authenticate_user_and_return!
		if user_signed_in?
			restore_params
			return
		end

		session_save_return

		authenticate_user!
	end

	# Проверяем, не нужно ли нам восстановить параметры.
	# Этот метод стоит вызывать через before_action или явным образом в тех методах, где
	# используется авторизация внутри. Например, если какой-то action из нескольких шагов
	# потребуется на промежуточном шаге авторизацию, ему нужно будет самостоятельно
	# проверять и восстанавливать параметры, сохраняемые во время авторизации.
	def restore_params
		# Restore post content after returning after authentication

		if session.key? :res_post
			cookie_enable

			# Сохраняем восстановленный POST в текущей обработке контроллера,
			# чтобы можно было его снова сохранить при session_save_return
			@session_res_post = session.delete :res_post

			self.params.merge! @session_res_post
		end
	end

	def session_return?
		session.key? :return_stack
	end

	# Сохраняем контекст - адрес и параметры (POST,GET) для того,
	# чтобы можно было на них вернуться
	def session_save_return uri = nil
		cookie_enable

		uri ||= request.original_fullpath
		post = if request.post?
				request.request_parameters
			elsif session.key? :res_post
				session.delete :res_post
			else
				@session_res_post
			end

		# Save current uri to return after authentication
		(session[:return_stack] ||= []) << [uri, post]
	end

	# Забираем сохраненный URL и подготавливаем для восстановления
	# сохраненные параметры.
	def session_take_return
		return unless session.key? :return_stack

		cookie_enable

		list = session[:return_stack].pop
		session.delete(:return_stack) if session[:return_stack].empty?

		if list[1]
			session[:res_post] = list[1]
		else
			session.delete :res_post
		end
		list[0]
	end

	def session_clear_return
		cookie_enable
		session.delete :return_stack
	end

end
