class Auth < HyperMuiComponent

	before_mount do
		# Уточняем у сервера авторизован ли юзер.
		# Если current_user nil, то мы не знаем его статус
		# Если false - то проверено, что не авторизован
		# Если объект - то авторизован
		# LATER нужно еще в куках проверять.
		if AppState.current_user.nil?
			AuthService.reload
		end
	end

  render do
		if AppState.current_user
			Authenticated(display: AppState.current_user.email)
		else
			Anonymous()
		end
  end

	class Anonymous < HyperMuiComponent
		render do
			LinkPopover(title: 'Log in'.m) { Panel() }
		end
	end

	class Panel < HyperMuiComponent
		param :panel, default: :login

		before_mount do
			@panel = panel
		end

		render(DIV) do
			case @panel
			when :login
				Typography(variant: :h6) { 'Login' }
				Auth::Login().on(:switch) {|value| mutate @panel = value}
			when :register, nil
				Typography(variant: :h6) { 'Register' }
				Auth::Register().on(:switch) {|value| mutate @panel = value}
			when :reset
				Typography(variant: :h6) { 'Reset password' }
				Auth::Reset().on(:switch) {|value| mutate @panel = value}
			end
		end
	end

	class Authenticated < HyperMuiComponent
		class Wrapped < HyperMuiComponent
			include Hyperstack::Router::Helpers
			param :display
			before_mount do
				@anchor = nil
			end
			def close
				mutate @anchor = nil
			end

			render(Fragment) do
				Button(color: :inherit) { display }
				.on(:click){|e| mutate @anchor = e.current_target }

				Menu(
						open: !@anchor.nil?,
						anchorEl: @anchor&.to_n,
						transformOrigin: { vertical: :bottom, horizontal: :left}.to_n,
						keepMounted: true
					) {
						MenuItem { 'Change email' }.on(:click) { history.push('/lomt/auth/change_email'); close }
						MenuItem { 'Change password' }.on(:click) { history.push('/lomt/auth/change_password'); close }
						MenuItem { 'Log out' }.on(:click) { AuthService.logout; close }
					}
				.on(:close){ close }
			end
		end

		imports `ReactRouter.withRouter(#{Wrapped.to_n})`
	end

	class Login < HyperMuiComponent
		fires :switch
		before_mount do
			@errs = {}
		end
		render do
			forgot_link = Link() { 'Forgot password?' }.on(:click) { switch! :reset }.as_node
			FORM do
				FormGroup do
					TextField name: 'email', label: 'Email', required: true, inputRef: catch_ref(:email),
						error: !@errs[:email].blank?, helperText: @errs[:email] || ''
					TextField type: 'password', name: 'password', label: 'Password', required: true,
						helperText: [forgot_link].to_n,
						inputRef: catch_ref(:password)

					P { }

					Button(type: 'submit', variant: :contained, color: :primary) { 'Log in' }
					Button() { 'Register' }.on(:click) { switch! :register }
				end
			end.on(:submit) do |evt|
				evt.prevent_default
				form = @refs.transform_values {|ref| Native(ref).value }
				AuthService.login_email(form)
					.fail{|err|
						message = [:email, :password].map{|key| err.errors.symbolic[key]}.compact.map(&:m).join('; ')
						mutate @errs[:email] = message
					}
			end
		end
	end # class Login

	class Register < HyperMuiComponent
		fires :switch
		before_mount do
			@errs = {}
		end
		render do
			Container maxWidth: :xs do
				FORM do
					FormGroup do
						TextField name: 'email', label: 'Email', required: true, inputRef: catch_ref(:email),
							error: !@errs[:email].blank?, helperText: @errs[:email] || ''
						TextField type: 'password', name: 'password', label: 'Password', required: true, inputRef: catch_ref(:password),
							error: !@errs[:password].blank?, helperText: @errs[:password] || ''
						TextField type: 'password', name: 'repeat_password', label: 'Repeat password', required: true, inputRef: catch_ref(:repeat_password),
							error: !@errs[:repeat_password].blank?, helperText: @errs[:repeat_password] || ''
					end

					P { }

					FormGroup do
						Lab::PROFILE_FIELDS.each do |field|
							options = {name: field, label: Lab::TITLES[field], inputRef: catch_ref(field), error: !@errs[field].blank?, helperText: @errs[field] || ''}
							case Lab::TYPES[field]
							when :text_small
								options[:multiline] = true
								options[:rowsMax] = 4
							end
							TextField **options
						end
					end

					P { }

					FormGroup do
						Button(type: 'submit', variant: :contained, color: :primary) { 'Register' }
						Button() { 'Log in' }.on(:click) { switch! :login }
					end
				end.on(:submit) do |evt|
					evt.prevent_default
					form = @refs.transform_values {|ref| Native(ref).value }
					AuthService.register_email_and_lab(form)
						.then{|res|
							puts "#{self} then: #{res}"
						}
						.fail{|err|
							puts "#{self} fail: #{err}"
							errs = nil
							if err.is_a?(Hyperstack::Operation::ValidationException)
								errs = err.errors.symbolic.transform_values(&:m)
								puts "errs: #{errs}"
							end

							mutate @errs = errs unless errs.nil?
						}
				end
			end
		end
	end # class Register

	class Reset < HyperMuiComponent
		fires :switch
		before_mount do
			@errs = {}
		end
		render do
			FORM do
				FormGroup do
					TextField name: 'email', label: 'Email', required: true, inputRef: catch_ref(:email),
						error: !@errs[:email].blank?, helperText: @errs[:email] || ''

					P { }

					Button(type: 'submit', variant: :contained, color: :primary) { 'Reset' }
					Button() { 'Log in' }.on(:click) { switch! :login }
				end
			end.on(:submit) do |evt|
				evt.prevent_default
				form = @refs.transform_values {|ref| Native(ref).value }
				puts "form: #{form}"
				AuthService.reset_password(form)
					.then{|res|
						puts "#{self} then: #{res}"
					}
					.fail{|err|
						puts "#{self} fail: #{err}"
						errs = nil
						if err.is_a?(Hyperstack::Operation::ValidationException)
							errs = err.errors.symbolic.transform_values(&:m)
						end

						mutate @errs = errs unless errs.nil?
					}
			end
		end
	end # class Reset

	class LinkPopover < HyperMuiComponent
		param :title
		before_mount do
			@anchor = nil
		end
		render(Fragment) do
			Button(color: :inherit) { title }
			.on(:click){|e| mutate @anchor = e.current_target }

			Popover(
					open: !@anchor.nil?,
					anchorEl: @anchor&.to_n,
					anchorOrigin: {vertical: :bottom, horizontal: :left}.to_n,
					transformOrigin: { vertical: :top, horizontal: :left}.to_n,
				) {
					Box(p: '1rem') {
						children.each(&:render)
					}
				}
			.on(:close){ mutate @anchor = nil }
		end
	end

end

=begin
LATER
Принимать в Auth параметры:

- коллбэк, который возвращает объект юзера или nil (вместо зашитого User.current)
- имя поля или proc, которые дает текстовое имя аутентифицированного пользователя для вывода его в виджете

=end
