module Batch
	class View::Create < HyperMuiComponent
		fires :submitted
		fires :saved

		before_mount do
			# Для хранение значений формы используем как refs для uncontrolled, так и vals для controlled
			# При сборке формы данные сливаются вместе из обоих хэшей.
			@refs = {}
			@vals = {}

			@max_samples_number = nil
			@samples_number = nil
		end

		def field name, options = {}
			type = options.delete(:type) { :TextField }
			if len = options.delete(:characters)
				(options[:inputProps] ||= {})[:maxLength] ||= len
				(options[:style] ||= {})[:width] ||= "#{len*1.25}ch"
			end
			if options.delete(:numeric)
				((options[:inputProps] ||= {})[:style] ||= {})[:textAlign] ||= :right
				(options[:inputProps] ||= {})[:inputMode] ||= :numeric
				(options[:inputProps] ||= {})[:pattern] ||= "[0-9.,/ -]+"
			end
			options[:name] ||= name

			options[:defaultValue] ||= Native(@refs[name])&.value if @refs.key?(name)

			if options.key? :inputRef
				options.delete(:inputRef) if options[:inputRef].nil?
			else
				options[:inputRef] = ->(el){@refs[name] = el}
			end

			if popover = options.delete(:helperPopover)
				(options[:InputProps] ||= {})[:endAdornment] = PopoverHelp { popover }.as_node
			end

			options[:inputProps] = options[:inputProps].to_n if options[:inputProps]
			options[:InputProps] = options[:InputProps].to_n if options[:InputProps]

			send type, options
		end

		render do
			puts "#{self}#render"
			FORM do
				Box(display: :flex, flexDirection: :row) {
					Box(pr: '1rem', pb: '1rem') {
						field :tid, label: 'Batch ID'
					}
					Box(pr: '1rem', pb: '1rem') {
						FormControl {
							InputLabel(htmlFor: :samples_plate_select) { 'Plate' }
							Select(name: :samples_plate, id: :samples_plate_select, native: true) {
								OPTION(value: '')
								Plate::ARRANGEMENTS.each do |arr|
									OPTION(value: arr) { arr }
								end
							}
							.on(:change) {|evt|
								if evt.target.value.present?
									@vals[:samples_plate] = evt.target.value
									mutate @max_samples_number = Plate::size(evt.target.value)
								else
									@vals.delete :samples_plate
									mutate @max_samples_number = nil
								end

								if @max_samples_number
									@vals[:samples_number] = @max_samples_number
									mutate @samples_number = @max_samples_number
								end
							}
						}
					}
					Box(pr: '1rem', pb: '1rem') {
						# Если выбрана плашка, то используется contolled input для того, чтобы не давать указать
						# число сэмплов больше, чем позволяет плашка.
						if @max_samples_number
							field(:samples_number, key: :samples_number_contoller, label: "Samples (max #{@max_samples_number})", numeric: true, characters: 8, inputRef: nil, value: @samples_number)
							.on(:change) {|evt|
								if evt.target.value.to_i <= @max_samples_number.to_i
									@vals[:samples_number] = evt.target.value
									mutate @samples_number = evt.target.value
								end
							}
						else
							@vals.delete :samples_number
							field(:samples_number, key: :samples_number_uncontoller, label: 'Samples', numeric: true, characters: 8, defaultValue: @samples_number)
						end
					}
		#				field :samples, label: 'Sample IDs', multiline: true, rowsMin: 4, rowsMax: 16
					Box(pr: '1rem', pb: '1rem') {
						field :pool_limit, label: 'Max pool', numeric: true, characters: 8
					}
					Box(pr: '1rem', pb: '1rem') {
						Button(type: 'submit', variant: :contained, color: :primary, style: {marginTop: '0.4rem'}) { 'Create' }
					}
					Box(pr: '1rem', pb: '1rem') {
						IconButton(color: :primary){IconHelp()}
						.on(:click){|e| mutate @anchor_help = e.current_target }
						Popover(
								open: !@anchor_help.nil?,
								anchorEl: @anchor_help&.to_n,
								anchorOrigin: {vertical: :bottom, horizontal: :left}.to_n,
								transformOrigin: { vertical: :top, horizontal: :left}.to_n,
							) {
								Box(p: '1rem') { 
									List {
										ListItem {
											ListItemText primary: 'Batch ID', secondary: 'The identifier of the batch. Leave empty to autogenerate'
										}
										ListItem {
											ListItemText primary: 'Samples', secondary: 'The number of the samples to be tested in the batch'
										}
										ListItem {
											ListItemText primary: 'Pool size', secondary: 'The maximum number of samples that can be mixed in a single assay'
										}
									}
								}
							}
						.on(:close){ mutate @anchor_help = nil }
					}
				}
			end
			.on(:submit) do |evt|
				evt.prevent_default
				puts "on submit"
				# Оставляем только время, остальные значения обнуляем
				form = @refs.transform_values{|val| Native(val)&.value&.nullify(&:blank?) }.compact
				form.merge! @vals
				puts "form: #{form}"
				transient = Api.create form
				submitted! transient
			end # .on(:submit)
		end # render

		class PopoverHelp < HyperMuiComponent
			render do
				InputAdornment(position: :end){
					Icon(fontSize: :small, color: :action){'help'}
					.on(:click){|e| mutate @anchor = e.current_target }
					Popover(
							open: !@anchor.nil?,
							anchorEl: @anchor&.to_n,
							anchorOrigin: {vertical: :bottom, horizontal: :left}.to_n,
							transformOrigin: { vertical: :top, horizontal: :left}.to_n,
						) {
							Box(p: '1rem') { 
								children.each(&:render)
							}
						}
					.on(:close){ mutate @anchor = nil }
				}
			end
		end

	end
end

=begin
TODO запоминать где-то в cookie или в User.settings pool_limit
=end
