module Batch
	class View::ServerList < HyperMuiComponent

		PAGE_SIZE = 5

		# Должен вернуть TransientObservable
		def count
			Api.count
		end

		def list page, &block
			Api.list(offset: PAGE_SIZE*(page-1), limit: PAGE_SIZE, &block)
		end

		before_mount do
			puts "#{self} before_mount"
			@page = 1
			@cache = []
			@count_t = count
			@list = list(@page)
		end

		# В переключении страниц список обновляется асинхронно, после
		# того как он обновится в TransientObservable объект, отданном из Api.list
		# Для того, чтобы переключение страниц было мягким, до тех пор пока
		# не появился новый список для страницы, показывается предыдущий список.
		# Поэтому мы храним его в атрибуте объекта @list и меняем только после того,
		# как Transient объект оказался ready.
		render(DIV) do
			puts "#{self} render, @page: #{@page}"
			if @count_t.error?
				Typography(color: :error, variant: :h6) { 'Count error' }
				Typography(color: :error) { @count_t.error.to_s }
			elsif @list.error?
				Typography(color: :error, variant: :h6) { 'List error' }
				Typography(color: :error) { @list.error.to_s }
			elsif @count_t.ready? && @list.ready?
				pages = ((@count_t.value - 1) / PAGE_SIZE).to_i + 1

				H2 { "Count: #{@count_t.value}" }

				Batch::View.List collection: @list.value

				if pages > 1
					Pagination(count: pages, color: :primary)
						.on(:change) {|evt, new_page|
							@cache[@page] = @list.value if @list.ready? # сохраняем в кэше значение старой страницы
							if @cache[new_page].nil?
								list(new_page){@list}
							else
								@list.ready @cache[new_page]
							end
							@page = new_page
						}
				end
			else
				CircularProgress()
			end
		end



	end
end
