module Batch
	class View::Work < HyperMuiComponent
		include Hyperstack::Router::Helpers

		param :batch
		fires :started
		fires :deleted

		render(DIV) do
			puts "#{self} render"

			aspect = batch.value if batch.is_a?(ModelApi::Transient)
			next if aspect.nil?

			H2 {
				case aspect.state
				when :new; 'New batch'
				when :plan; "Stage #{aspect.depth} plan"
				when :assay; "Stage #{aspect.depth} waiting assay results"
				when :ready; "Stage #{aspect.depth} completed"
				end
			}

			# Общая информация о батче
			TABLE {
				TR {
					TD { "Batch:" }
					TD { aspect.tid }
				}

				TR {
					TD { "Samples:" }
					TD {
						case aspect.state
						when :new
							PopoverList title: "All samples", collection: aspect.all_samples
						when :finished
							SPAN { aspect.all_samples.size.to_s }
						else
							PopoverList title: "All samples", collection: aspect.all_samples
							SPAN { " (" }
							PopoverList title: "Processed samples with results", collection: Batch::boolean_to_plusminus(aspect.all_sample_results.to_a), headers: %w[Sample Result]
							SPAN { "+" }
							PopoverList title: "Samples waiting", collection: aspect.samples_waiting || []
							SPAN { "+" }
							PopoverList title: "Samples left", collection: aspect.all_samples_left
							SPAN { ")" }
						end
						SPAN { " / #{aspect.samples_plate}" } if aspect.samples_plate
					}
				}

				TR {
					TD { "Assays:" }
					TD {
						case aspect.state
						when :finished
							SPAN { aspect.all_assays.size.to_s }
						else
							PopoverList title: "All assays", collection: aspect.all_assays
							SPAN { " (" }
							PopoverList title: "Assay results", collection: Batch::boolean_to_plusminus(aspect.all_assay_results.to_a), headers: %w[Assay Result]
							SPAN { "+" }
							PopoverList title: "Assays waiting", collection: aspect.assays_waiting
							SPAN { ")" }
						end
					}
				} if aspect.state != :new

				TR {
					TD { "Operations:" }
					TD { aspect.plan.size.to_s }
				} if aspect.state == :plan

				TR {
					TD { "Planners:" }
					TD {
						if aspect.state == :finished
								aspect.all_planner_names.join(', ')
						elsif aspect.planner_names
								aspect.planner_names.join(', ')
						end
					}
				} if aspect.state != :new

				TR {
					TD { "Stages:" }
					TD { Link("/lomt/traces/#{aspect.id}") { aspect.stages.to_s } }
				} if aspect.stages && aspect.stages > 0

				TR {
					TD { "Created:" }
					TD { aspect.created_at.to_s }
				}
				TR {
					TD { "Changed:" }
					TD { aspect.updated_at.to_s }
				}
			}

			P { }

			case aspect.state
			when :new
				Button(variant: :contained, color: :primary) { 'Start' }
				.on(:click) {
					# После того как придет обновление в Transient batch от switch state: :plan,
					# мы отправляем ивент started! с id батча вызвавшему нас компоненту. Сейчас
					# это Index, который после получения started перенаправляет браузер через history.push
					# на страницу батча.
					batch.on_ready do
						started! aspect.id
					end
					Api.switch(state: :plan){batch}
				}
			when :plan
				MixingPlan::Home(plan: aspect.plan.map{|op| op.values_at :sample, :assay}, samples_plate: aspect.samples_plate)
				.on(:done) {
					Api.switch(state: :assay){batch} 
				}

				P { }

				Button(variant: :contained, color: :primary) { 'Done' }
				.on(:click) { Api.switch(state: :assay){batch} }
			when :assay
				if aspect.assay_results && aspect.assay_results.size > 0
					TableList title: "Assay results", headers: %w[Assay Result], collection: Batch::boolean_to_plusminus(aspect.assay_results.to_a)

					P { }
				end

				refs = {}
				FORM {
					if aspect.assays_waiting && aspect.assays_waiting.size > 0
						TABLE {
							THEAD {
								TR { TH(rowSpan: 2) { 'Assay' }; TH(colSpan: 3) { 'Result' } }
								TR { TH { 'Waiting'}; TH { 'Negative' }; TH { 'Positive' } }
							}
							TBODY {
								aspect.assays_waiting.each do |assay|
									TR {
										TD { assay }
										%w[nil false true].each do |value|
											TD {
												INPUT key: assay, type: :radio, name: assay, value: value, ref: ->(el){(refs[assay] ||= {})[value] = el}
											}
										end
									}
								end
							}
						}
					end

					P { }

					Button(variant: :contained, color: :primary, type: :submit) { 'Submit' }
				}.on(:submit) { |evt|
					evt.prevent_default
					value_map = {'nil' => nil, 'false' => false, 'true' => true }
					assay_results = {}
					refs.each do |name, value|
						if value.is_a? Hash
							value.each do |value, el|
								if Native(el.dom_node).checked
									assay_results[name] = value_map[value]
								end
							end
						end
					end
					Api.report(report: assay_results){batch}
				}
			when :ready
				TableList title: "Assay results", headers: %w[Assay Result], collection: Batch::boolean_to_plusminus(aspect.assay_results.to_a)

				P { }

				TableList title: "Sample results", headers: %w[Sample Result], collection: Batch::boolean_to_plusminus(aspect.sample_results.to_a)

				P { }

				TableList title: "Samples left", collection: aspect.all_samples_left

				P { }

				if aspect.samples_left && aspect.samples_left.size > 0
					Button(variant: :contained, color: :primary) { 'Next' }
					.on(:click) { Api.switch(state: :plan){batch} }
				else
					Button(variant: :contained, color: :primary) { 'Finish' }
					.on(:click) { Api.switch(state: :finished){batch} }
				end
			when :finished
				TextareaList title: "Processed samples with results", headers: %w[Sample Result], collection: Batch::boolean_to_plusminus(aspect.all_sample_results.to_a)

				P { }

				TextareaList title: "Assay results", headers: %w[Assay Result], collection: Batch::boolean_to_plusminus(aspect.all_assay_results.to_a)

			end

			P { }

			Button() { 'Delete' }
			.on(:click){|e| mutate @anchor_delete = e.current_target }

			Popover(
					open: !@anchor_delete.nil?,
					anchorEl: @anchor_delete&.to_n,
					anchorOrigin: {vertical: :bottom, horizontal: :left}.to_n,
					transformOrigin: { vertical: :top, horizontal: :left}.to_n,
				) {
					FORM {
						Box(p: '1rem') { 
							P(align: :center) { 'Are you sure?' }
							Box(display: :flex, flexDirection: :row) {
								Box(px: '1rem') {
									Button(variant: :contained, color: :primary, type: :submit) { 'Yes' }
								}
								Box(pr: '1rem') {
									Button(variant: :contained, color: :primary) { 'No' }
									.on(:click) { mutate @anchor_delete = nil }
								}
							}
						}
					}.on(:submit) { |evt|
						evt.prevent_default
						Api.delete(id: aspect.id){
							ModelApi::Transient.new.on_ready do |tr|
								deleted! tr.value
							end
						}
					}
				}
			.on(:close){ mutate @anchor_delete = nil }


		end
	end
end
