class LomtApp < HyperMuiComponent
	include Hyperstack::Router

	LOMT_VERSION = '0.9'
	FOOTER_HEIGHT = '80px'
	EMAIL = 'LOMTService@gmail.com'

	before_mount do
		@drawer_open = false
	end

	DEFAULT_THEME = `Mui.createMuiTheme()`

  render(Fragment) do
		ThemeProvider(theme: DEFAULT_THEME) {
			CssBaseline()


			DIV(style: {minHeight: '100vh'}) {
				AppBar position: :static do
					Toolbar do
						IconButton edge: :start, color: :inherit, "aria-label": "menu" do
							IconMenu()
							.on(:click) { mutate @drawer_open = !@drawer_open }
						end
						Typography variant: :h6, style: {flexGrow: 1} {
							'LOMT'
						}
						Auth()
					end
				end

				# Здесь мы оборачиваем в Route / только для того, чтобы получить
				# history и передать затем в MainDrawer. Ему history нужна для history.push
				# при поиска батчей. А оборачивать его в withRouter мы не можем, т.к. тогда
				# из него не работает передача ивента close.
				Route '/' do |match, location, history|
					MainDrawer(open: @drawer_open, history: history)
					.on(:close) { mutate @drawer_open = false }
				end

				Paper() {
					Box(p: '1rem') {

						Switch {
							Route '/lomt/auth/', component: Auth::Page.to_n

							puts "AppState.current_user: #{AppState.current_user} #{AppState.current_user.class}"
							Route('/lomt/', exact: true) do
								DIV {
									if AppState.current_user
										P(style: {textAlign: :center}) {
											Button(variant: :contained, color: :primary, component: `ReactRouterDOM.Link`, to: '/lomt/batches') { 'Open the batches workplace' }
										}
									elsif AppState.current_user == false
										Container(maxWidth: :xs) {
											Auth::Panel(panel: :register)
										}
									end
									# если про авторизацию неизвестно (nil), то значит скоро будет известно,
									# когда Auth отработает загрузку. А пока ничего не показываем, чтобы не мелькать.
									DIV(dangerously_set_inner_HTML: { __html: Content::INTRO })
								}
							end

							if AppState.current_user
								Route('/lomt/batches', exact: true) do |match, location, history|
									Batch::View::Index()
								end

								Route '/lomt/batches/:id' do |match, location, history|
									DIV {
										Batch::View.Work(batch: Batch::Api.read(id: match.params[:id]))
										.on(:deleted) { |batch| history.push("/lomt/batches/") }
										P {
											Link('/lomt/batches') { 'Return to the list' }
										}
									}
								end

								Route '/lomt/traces/:id' do |match, location, history|
									DIV {
										Batch::View::Traces batch_id: match.params[:id]
										P {
											Link('/lomt/batches') { 'Return to the list' }
										}
									}
								end
							end

						}

					}
				}


				DIV(style: {height: FOOTER_HEIGHT, marginTop: '1rem'}) { }
			}


			DIV(style: {clear: :both, position: :relative, margin: "-#{FOOTER_HEIGHT} 1rem 0 1rem", height: FOOTER_HEIGHT}) {
				DIV { "Laboratory Optimizer for Mass Testing v#{LOMT_VERSION}" }
				DIV(dangerously_set_inner_HTML: { __html: "&copy; 2020 Kirill Vechera, Jetware SRL (Rome, Italy), OOO ServerSnab (Saint-Petersburg, Russia)"})
				DIV { SPAN{'Contacts: '}; A(href: "mailto:#{EMAIL}") { EMAIL} }
			}
		}
  end # render


	class MainDrawer < HyperMuiComponent
		include Hyperstack::Router::Helpers
		fires :close
		param :open
		param :history

		before_mount do
			@drawer_open = false
			@find_batch = nil
		end

		render(Fragment) do
			Drawer(open: open) {
				Paper {
					Divider()
					List {
						ListItem(button: true, component: `ReactRouterDOM.Link`, to: '/lomt/' ) {
							ListItemText primary: 'Intro'
						}.on(:click) { close! }

						ListItem(button: true, component: `ReactRouterDOM.Link`, to: '/lomt/batches' ) {
							ListItemText primary: 'Batches'
						}.on(:click) { close! }
					}

					Divider()

					List {
						ListItem {
							FORM {
								FormGroup(row: true) {
									TextField name: 'tid', label: 'Batch ID', style: {width: "#{10*1.25}ch"}, inputRef: catch_ref(:tid), helperText: 'Go to batch', autoFocus: true
									IconButton(type: 'submit', color: :primary){IconSend()}
								}
							}
							.on(:submit) do |evt|
								evt.prevent_default
								form = @refs.transform_values {|ref| Native(ref).value }
								if form[:tid].present?
									mutate @find_batch = true
									Batch::Api.find(tid: form[:tid]) {
										ModelApi::Transient.new.on_ready do |tr|
											if tr.value
												history.push("/lomt/batches/#{tr.value}")
												mutate @find_batch = nil
												close!
											else
												mutate @find_batch = "Batch #{form[:tid]} not found"
											end
										end
									}
								end
							end
						}
						if @find_batch
							if @find_batch.is_a? String
								Box(p: '1rem') {
									Typography(color: :error, variant: :h6) { 'Error' }
									Typography(color: :error) { @find_batch }
								}
							else
								CircularProgress()
							end
						end
					}

					Divider()

					List {
						ListItem(component: A.to_n, href: '/') { 'Website' }
					}.on(:click) { close! }

				}
			}.on(:close) { close! }
		end

	end # class MainDrawer

end

