module MixingPlan::Directions
	class Base
		include Hyperstack::State::Observable

		attr_reader :map, :direct
		def initialize map, direct
			@map, @direct = map, direct
			@step = nil
			@sources = @map.keys
			@targets = @map.values
			@count = @sources.size
			@curr_samples = @curr_assays = Array::EMPTY
			@done_samples = @done_assays = Array::EMPTY
			@left_samples = @left_assays = Array::EMPTY
		end
		attr_reader :step, :count

		def start
			@step = 0
			update
		end

		def backward
			return if !started? || @step <= 0
			@step -= 1
			update
		end

		def forward
			return if !started? || @step >= @count - 1
			@step += 1
			update
		end

		def end
			@step = @count - 1
			update
		end

		def finish
			@step = @count
			clear
		end

		def select sample, assay
			raise NotImplementedError
		end

		def update
			raise NotImplementedError
		end
		attr_reader :curr_samples, :curr_assays
		attr_reader :left_samples, :left_assays
		attr_reader :done_samples, :done_assays

		def started?
			observe { !@step.nil? }
		end

		def enroute?
			observe { started? && @step < @count - 1 }
		end

		def finished?
			observe { started? && @step >= @count }
		end

		def reset
			@step = nil
			clear
		end

		def clear
			@curr_samples = @curr_assays = Array::EMPTY
			@done_samples = @done_assays = Array::EMPTY
			@left_samples = @left_assays = Array::EMPTY
			mutate
		end

		def view
			self.class::View(map: @map, direct: @direct, line: @step)
		end

		class View < HyperMuiComponent
			fires :selected
			param :map, type: Hash
			param :direct
			param :line, default: nil, nils: true

			render(TABLE) do
				THEAD { TR { TH { 'Sample' }; TH { 'Assay' } } }
				i = 0
				TBODY {
					map.each do |source, targets|
						TR(i == line ? {style: {fontWeight: :bold}} : Hash::EMPTY) {
							case direct
							when :sample
								TD {
									SPAN { source.join }
									.on(:click) { selected! source, targets.first }
								}
								TD {
									targets.each do |target|
										SPAN { "#{target.join} " }
										.on(:click) { selected! source, target }
									end
								}
							when :assay
								TD {
									targets.each do |target|
										SPAN { "#{target.join} " }
										.on(:click) { selected! target, source }
									end
								}
								TD {
									SPAN { source.join }
									.on(:click) { selected! targets.first, source }
								}
							end
						}
						i += 1
					end
				}
			end
		end

	end
end
