module Batch

	DEFAULT_STRATEGY = 'LTO::Strategy::PoolLinear::SIMPLE'

	# Эти состояния несколько отличаются от состояний LTO::Batch, потому что здесь
	# описаны состояния для пользовательского интерфейса Batch, а там описаны состояния,
	# которые отражают состояние процесса и участвуют в логике методов класа.
	# В LTO::Batch не определяются явно состояния plan и assay, и вообще не
	# отделяются как два разных состояния.
	STATES = {
		new:       0, # the batch is just created
		plan:   1, # a stage plan created
		assay:   2, # a stage is waiting for assay results
		ready:     3, # a stage is ready with all assay results received and sample results are inferred
		finished:  4, # the batch is finished with all stages completed and no samples left
	}




	def boolean_to_plusminus value
		case value
		when Hash
			value.transform_values &self.method(__callee__)
		when Array
			value.map &self.method(__callee__)
		when true, 'true', 1, '1'
			'+'
		when false, 'false', 0, '0'
			'-'
		when nil, 'nil'
			'?'
		else
			value
		end
	end
	module_function :boolean_to_plusminus

end
