class AuthService

	class UserApi < ModelApi::Api
		model :User
		aspect :Email, %i[id email]
	end

	def self.login_email **args
		LoginEmail.run(**args)
			.then{ |res|
				puts "res: #{res}, res.class: #{res.class}"
				AppState.current_user = res
			}
	end

	def self.register_email **args
		RegisterEmail.run(**args)
			.then{ |res|
				raise res if res.is_a?(Hyperstack::Operation::ValidationException) # из-за бага в Operation, когда ошибка возвращается в then
				AppState.current_user = res
			}
	end

	def self.register_email_and_lab **args
		RegisterEmailAndLab.run(**args)
			.then{ |res|
				raise res if res.is_a?(Hyperstack::Operation::ValidationException) # из-за бага в Operation, когда ошибка возвращается в then
				AppState.current_user = res
		}
	end

	def self.logout
		Logout.run
			.then{ |res|
				puts "#{self} then #{res}"
				AppState.current_user = false
			}
	end

	def self.reload
		Reload.run()
			.then{ |res|
				puts "reload. res: #{res}, res.class: #{res.class}"
				AppState.current_user = res || false
			}
	end

	class LoginEmail < Hyperstack::ControllerOp
		param :email
		inbound :password

		include ModelApi::HyperstackTransport

		unless RUBY_ENGINE == 'opal'
			add_error(:email, :email_does_not_exist, 'login does not exist') { !(@user = User.find_by_email(params.email)) and abort! }
			add_error(:password, :password_is_incorrect, 'password is incorrect') { !@user.valid_password?(params.password)  }
			step { puts "user already signed in #{current_user}" if user_signed_in? }
			step {
				user =  user_signed_in? ? current_user : sign_in(:user, @user)
				puts "user: #{user}"
				UserApi::AspectEmail.from_model(user).tap do |x|
					puts "user aspect: #{x}"
				end
			}
		end
	end

	class RegisterEmail < Hyperstack::Operation
		param :email
		inbound :password
		inbound :repeat_password

		validate do
			add_error(:email, :invalid_email, 'invalid email address') unless params.email =~ /\w@\w/
			add_error(:password, :empty_password, 'password not set') unless params.password.size > 0
			add_error(:password, :empty_repeat_password, 'repeat password not set') unless params.repeat_password.size > 0
			add_error(:repeat_password, :doesnt_match, 'password doesn\t match') unless params.password == params.repeat_password
			true
		end

		class OnServer < Hyperstack::ControllerOp
			param :email
			inbound :password

			include ModelApi::HyperstackTransport

			validate do
				if User.find_by_email(params.email)
					add_error(:email, :email_already_taken, 'email already taken')
					abort!
				end

				@user = User.new(email: params.email, password: params.password)
				unless @user.valid?
					@user.errors.each do |attr, message|
						add_error(attr, message.to_sym, message)
					end
					abort!
				end

				true
			end

			step {
				if @user.save
					sign_in(:user, @user)
					UserApi::AspectEmail.from_model(@user)
				else
					fail 'cannot save the record'
				end
			}
		end

		step OnServer

	end # class RegisterEmail

	class RegisterEmailAndLab < Hyperstack::Operation
		param :email
		inbound :password
		inbound :repeat_password
		Lab::PROFILE_FIELDS.each do |field|
			param field, default: nil, nils: true
		end

		validate do
			add_error(:email, :invalid_email, 'invalid email address') unless params.email =~ /\w@\w/
			add_error(:password, :empty_password, 'password not set') unless params.password.size > 0
			add_error(:password, :empty_repeat_password, 'repeat password not set') unless params.repeat_password.size > 0
			add_error(:repeat_password, :doesnt_match, 'password doesn\t match') unless params.password == params.repeat_password
			true
		end

		class OnServer < Hyperstack::ControllerOp
			param :email
			inbound :password
			Lab::PROFILE_FIELDS.each do |field|
				param field, default: nil, nils: true
			end

			include ModelApi::HyperstackTransport

			validate do
				if User.find_by_email(params.email)
					add_error(:email, :email_already_taken, 'email already taken')
					abort!
				end

				@user = User.new(email: params.email, password: params.password)
				unless @user.valid?
					@user.errors.each do |attr, message|
						add_error(attr, message.to_sym, message)
					end
				end

				@lab = Lab::Model.new params.to_h.slice(*Lab::PROFILE_FIELDS)
				unless @lab.valid?
					@lab.errors.each do |attr, message|
						add_error(attr, message.to_sym, message)
					end
				end

				abort! unless @user.valid? && @lab.valid?

				true
			end

			step {
				@user.labs << @lab
				if @user.save
					sign_in(:user, @user)
					UserApi::AspectEmail.from_model(@user)
				else
					fail 'cannot save the record'
				end
			}
		end

		step OnServer

	end # class RegisterEmailAndLab


	class Logout < Hyperstack::ControllerOp
		unless RUBY_ENGINE == 'opal'
			step { sign_out(:user) }
		end
	end

	class Reload < Hyperstack::ControllerOp
		include ModelApi::HyperstackTransport

		unless RUBY_ENGINE == 'opal'
			step {
				if user_signed_in?
					UserApi::AspectEmail.from_model(current_user)
				else
					nil
				end
			}
		end
	end

end
