module Batch
	class Api < ModelApi::Api

		model 'Batch::Record'

		aspect :Item, %i[id tid all_samples created_at updated_at]

		# Получить дефолтные рабочий аспект объекта с сервера
		action :work do
#			aspect %i[id tid samples pool_limit sample_results assay_results samples_waiting assays_waiting samples_left plan]
			aspect %i[id state tid samples_plate all_samples pool_limit depth stages plan planner_names assays assay_results assays_waiting samples sample_results samples_waiting samples_left all_planner_names all_sample_results all_samples_left all_assays all_assay_results created_at updated_at]
			inbound :id
			set :observable

			run do |args|
				Batch::Record.find(args[:id])
			end
		end


		action :create do
			aspect :Work
			param :tid, default: nil, nils: true
			param :samples_plate, default: nil, nils: true
			param :samples_number, default: nil, nils: true
			param :samples, default: [], nils: true
			param :pool_limit
#			param :estimated_frequence

			# Отдается единственный объект
			set :observable do
				start do |args|
					puts "#{self}#start #{args}"
					attrs = args.first

					if attrs[:samples].blank? && attrs[:samples_number].present?
						attrs[:samples] = attrs[:samples_number].to_i
						attrs.delete :samples_number
					end

					abort 'samples number must be present' if attrs[:samples].blank?

					attrs_copy = attrs.dup
					attrs_copy[:all_samples] = attrs_copy[:samples].is_a?(Enumerable) ? attrs_copy[:samples] : ['']*attrs_copy[:samples]
					attrs_copy[:all_sample_results] = {}
					attrs_copy[:all_samples_left] = attrs_copy[:all_samples]

					AspectWork.from_hash(attrs_copy)
				end

			end

			run do |args|
				raise 'Non-authenticated user' unless current_user

				attrs = args
						.symbolize_keys
						.slice(:tid, :samples, :pool_limit, :samples_plate)
						.merge(strategy_class: DEFAULT_STRATEGY, user: current_user)
				attrs[:pool_limit] &&= attrs[:pool_limit].to_i
				attrs[:samples] = attrs[:samples].to_i if attrs[:samples].is_a? String

				r = Record.create! **attrs
			end
		end

		# Для switch и report если id не передан в качестве аргумента, то
		# мы пытаемся взять его из объекта внтури Transient-контейнера - того
		# объекта, который будет заменен в результате
		action :switch do
			aspect :Work
			inbound :id
			inbound :state

			set :observable do
				start do |args|
					args.first[:id] ||= @value&.id
				end
			end

			run do |args|
				raise 'Non-authenticated user' unless current_user

				current_user.batches.find(args[:id]).tap do |b|
					b.state = args[:state]
					b.save
				end
			end
		end

		action :report do
			aspect :Work
			inbound :id
			inbound :report

			set :observable do
				start do |args|
					args.first[:id] ||= @value&.id
				end
			end

			run do |args|
				raise 'Non-authenticated user' unless current_user

				current_user.batches.find(args[:id]).tap do |b|
					b.report args[:report]
					if b.batch.ready?
						b.state = :ready
					end
					b.save
				end
			end
		end

		action :read do
			aspect :Work
			inbound :id

			set :observable
			run do |args|
				raise 'Non-authenticated user' unless current_user
				current_user.batches.find(args[:id])
			end
		end

		action :find do
			inbound :tid

			set :observable
			run do |args|
				raise 'Non-authenticated user' unless current_user
				current_user.batches.where(tid: args[:tid]).pluck(:id).first
			end
		end

		action :delete do
			aspect :Work
			inbound :id

			set :observable
			run do |args|
				raise 'Non-authenticated user' unless current_user
				current_user.batches.find(args[:id]).destroy
			end
		end

		action :traces do
			aspect %w[id tid traces]
			inbound :id

			set :observable do
				start do |args|
					@value ||= AspectTraces.from_hash id: args.first[:id], tid: '', traces: []
				end
			end

			run do |args|
				raise 'Non-authenticated user' unless current_user
				current_user.batches.find(args[:id])
			end
		end



		action :count do
			set :observable

			run do |args|
				raise 'Non-authenticated user' unless current_user
				current_user.batches.all.count
			end

		end

		action :list do
			aspect %i[id tid state depth all_samples_count all_samples_left_count samples_waiting_count all_assays_count assays_waiting_count created_at updated_at]


			param :offset, default: nil, nils: true
			param :limit, default: nil, nils: true

			set :observable do
				start do |args|
					@value ||= []
				end
			end

			run do |args|
				raise 'Non-authenticated user' unless current_user
				list = current_user.batches.all.order(id: :DESC)
				list = list.offset(args[:offset]) if args[:offset]
				list = list.limit(args[:limit]) if args[:limit]
				list
			end

		end


	end
end



=begin

LATER

Пока создаем все Batches с DEFAULT_STRATEGY, но позже будем предлагать пользователю выбор
или автоматически определять стратегию в зависимости от указанных при создании параметров.

=end
