class ArrayStore
	include Hyperstack::State::Observable

	NEW_OR_INPLACE = %i[collect compact filter select reject map sort sort_by reverse shuffle rotate slice]

	MUTATING = %i[<< []= concat replace append delete insert pop push shift unshift clear]
	OBSERVING = %i[find each each_index each_with_index size [] first last present? blank? empty? any? all? none? include?]

	def initialize *args, &block
		@array = Array.new *args, &block
	end

	MUTATING.each do |method|
		define_method method do |*args, &block|
			mutate { @array.send(method, *args, &block) }
		end
	end

	OBSERVING.each do |method|
		define_method method do |*args, &block|
			observe { @array.send(method, *args, &block) }
		end
	end

	NEW_OR_INPLACE.each do |method|
		define_method method do |*args, &block|
			observe { self.class.new(@array.send(method, *args, &block)) }
		end

		inplace_method = "#{method}!".to_sym
		define_method inplace_method do |*args, &block|
			mutate { @array.send(method, *args, &block) }
		end
	end

	def dup
		observe { self.class.new @array.dup }
	end

	def clone
		observe { super.instance_exec{ @array = @array.clone; self }  }
	end

	def method_missing method_name, *args, &block
		@array.send method_name, *args, &block
	end

	def respond_to_missing? *args
		@array.respond_to_missing? *args
	end

end
