module DebugCall

	
	def self.to_s args
		return nil if args.nil?
		res = args.is_a?(Array) ? args.flatten.map(&:to_s).join(', ') : args.to_s
		res.gsub /[^[:print:]]/, '.'
	end

	def self.inspect args
		args.inspect
	end

	def self.call klass, *methods
		options = { self: :to_s, args: :to_s, result: :to_s, thread: false }
		options.merge!(methods.pop) if methods.last.is_a?(Hash)
		klass.class_exec do
			methods.each do |method|
				origin_method = "without_debug_call_#{method}".intern
				alias_method origin_method, method
				define_method(method) do |*args, &block|
					prefix = if options[:thread]
						tc = Thread.current
						"#{tc.class.name}\##{sprintf("%08x", tc.object_id)} "
					else
						''
					end

					printed_self = if printer = options[:self]
						printer.is_a?(Proc) ? printer.call(self) : DebugCall::send(printer, self)
					else
						self.class.to_s
					end
					
					msg = ">#{method}@#{printed_self}"
					if printer = options[:args]
						val_out = printer.is_a?(Proc) ? printer.call(args) : DebugCall::send(printer, args)
						msg += ": #{val_out}" unless val_out.nil?
					end
					puts prefix + msg
					begin
						msg = "<#{method}@#{printed_self}"
						result = send(origin_method, *args, &block)
						if printer = options[:result]
							val_out = printer.is_a?(Proc) ? printer.call(result) : DebugCall::send(printer, result)
							msg += ": #{val_out}" unless val_out.nil?
						end
						puts prefix + msg
						result
					rescue Exception => ex
						msg += ": Exception #{ex}"
						puts prefix + msg
						raise ex
					end
				end
			end
		end
	end

end
