require 'ruby_helper'
require 'lto/batch/front'
require 'lto/batch/trace'
module LTO
	class Batch

=begin

Life cycle:

* create a Batch object
* вызываем start, и он в ответ дает план операций
* вызывается report, в котором даются результаты тестов

CURR
* после каждого report нужно смотреть ready?
* если reported?, то запрашивать sample_results
* и потом выполнять next - создается следующий stage куда добавляются все новые планировщики, и снова plan и т.д.



=end

		include InstanceVariablesEqual

		# Класс для сбора результатов операций по типу - нормальный результат, предупреждения, ошибки
		class OperationReturn

			attr_accessor :results, :warnings, :errors
			def initialize results: nil, warnings: nil, errors: nil
				@results, @warnings, @errors = results, warnings, errors
			end

		end

		attr_reader :id, :strategy, :all_samples
		def initialize id, strategy_class, samples, pool_limit: 
			unless samples.is_a? Enumerable
				prefix = id.nil? ? "S" : "#{id}/S"
				samples = (1..samples).map{|i| "#{prefix}#{i}" }
			end

			strategy_class = Kernel.const_get(strategy_class) unless strategy_class.is_a? Class
			@strategy    = strategy_class.new pool_limit
			@id          = id
			@all_samples = samples
			@front       = nil
			@traces      = []
		end
		attr_reader :traces, :front

		def pool_limit; @strategy.pool_limit; end

		def start!
			return false if running? || finished?
			@front = Front.new(@strategy.planners nil, @all_samples)
			true
		end

		def forward!
			return false if !running?
			@traces << Trace.new(@front)
			@front = @front.forward
			true
		end

		def plan
			@front&.plan
		end

		def report results
			@front&.report results
		end

		def depth
			@traces.size + (running? ? 1 : 0)
		end

		# Если работы ведутся
		def running?
			!@front.nil?
		end

		# Если работы текущего этапа завершены и можно продолжать
		def ready?
			@front && @front.ready?
		end

		def resolved?
			@front && @front.samples_left.empty?
		end

		def finished?
			!@front && @traces.size > 0
		end

		def assays
			@front&.assays
		end

		def assay_results
			@front&.assay_results
		end

		def assays_waiting
			@front&.assays_waiting
		end

		def samples
			@front&.samples
		end

		def sample_results
			@front&.sample_results
		end

		def samples_waiting
			@front&.samples_waiting
		end

		def samples_left
			@front&.samples_left
		end

		def planner_names
			@front&.planner_names
		end

		def all_assays
			ret = @traces.map(&:assays).reduce([], :+)
			ret.concat(@front.assays) if @front
			ret
		end

		# Финальные результаты сэмплов, только готовые сэмплы
		def all_sample_results
			ret = @traces.map(&:sample_results).reduce({}, :merge)
			ret.merge!(@front.sample_results) if @front
			ret
		end

		# Финальные результаты тестов
		def all_assay_results
			ret = @traces.map(&:assay_results).reduce({}, :merge)
			ret.merge!(@front.assay_results) if @front
			ret
		end

		def all_samples_left
			if @front
				@front.samples_left
			elsif finished?
				Array::EMPTY
			else
				@all_samples
			end
		end

		def all_planner_names
			ret = @traces.map(&:planner_names).reduce([], :+)
			ret.concat(@front.planner_names) if @front
			ret.uniq
		end


		def dump_init
			{
				strategy_class:  @strategy.class.name,
				pool_limit:      @strategy.pool_limit,
				id:              @id,
				all_samples:     @all_samples,
				front:           @front&.dump_init,
				traces:          @traces.map(&:dump_init)
			}.compact
		end

		def self.load_init hash
			hash.transform_keys!(&:to_sym)
			obj = self.new hash[:id], Kernel.const_get(hash[:strategy_class]), hash[:all_samples], pool_limit: hash[:pool_limit]

			obj.instance_variable_set(:@front, Front.load_init(hash[:front])) if hash[:front]
			obj.instance_variable_set(:@traces, hash[:traces].map{|h| Trace.load_init(h)}) if hash[:traces]

			obj
		end
		alias init_eql? eql?

	end # class Batch
end # module LTO

=begin

TODO тест сериализации
	нужно проверять результат действия на оригинальном и сериализованном объекте
	результаты сверять проще всего в виде сериализации
	заготовка: spec/lto/batch/serialization_checkpoints_spec.rb

=end
