require 'lto/planner'
module LTO
	module Planner

		class PoolBisector < Base
			SECTION_FACTOR = 2

			# При бисекции лимит пула из стратегии с каждым новым шагом уменьшается в два раза, поэтому
			# хранится в планировщие и передается в следующие планировщики

			principal_attributes :pool_size
			def initialize strategy, samples, pool_size: strategy.pool_limit, batch: [], min_positives: 0
				super strategy, samples, batch: batch, min_positives: min_positives
				@pool_size = pool_size
				build_plan
			end

			def build_plan
				groups_count = ((@samples.size - 1) / @pool_size) + 1
				@assays = request_assays groups_count
				@assay_samples = @assays.each_index.map do |i|
					group_start = i*@pool_size
					group_final = [group_start + @pool_size - 1, @samples.size - 1].min
					(group_start..group_final).to_a
				end
			end

			def infer
				results = Array.new @samples.size
				@assay_samples.each_with_index do |samples, assay|
					if samples.size == 1
						# Если группы из одного образца, то результат теста равен результату сэмпла - false, true, nil
						results[samples.first] = @assay_output[assay]
					else
						# Если группа из нескольких образцов, то только отрицательный результат является
						# результатом сэмпла. Другие результаты означают, что результат неизвестен - nil
						samples.each do |sample|
							results[sample] = @assay_output[assay] == false ? false : nil
						end
					end
				end
				results
			end

			def next_planners
				descendants = []
				@assay_samples.each_with_index do |samples_indices, assay|
					next if @assay_output[assay] == false	# не проверяем группы с отрицательным тестом
					next if @assay_output[assay] == true && samples_indices.size == 1 # не проверяет группы из одного элемента с положительным тестом
					pool_size = (@pool_size - 1) / SECTION_FACTOR + 1
					samples = samples_indices.map{|i| @samples[i]}
					# XXX Может быть не сообщать pool_size в strategy_planners, а исходить из того, что если
					# следующим окажется бисектор, чтобы он узнал у предыдущего планировщика размер пула сам.
					descendants.concat strategy_planners(samples, pool_size: pool_size, batch: @batch+[assay])
				end
				descendants
			end

		end # class PoolBisector
	end # module Planner
end # module LTO

=begin

build_plan

Не нужно балансировать группы по размеру, если сэмплы не делятся на группы с
полными пулами: для сбалансированных групп в худшем случае (multiple positives)
число тестов будет одинаковым, а в лучшем случае (single positive) число тестов
окажется больше, чем если группы будут включать маленькую группу с остатком.

=end
