require 'lto/planner'
require 'lto/planner/pool_bisector'
module LTO
	module Planner

		class PoolBisectorWaiting < PoolBisector


			def build_plan
				if @min_positives > 0
					@assays = request_assays 1
					# Если лимит пула больше, чем половина размера сэмплов, т.е. он не делит сэмплы пополам,
					# то уменьшает размер пула, чтобы он поделил делил как можно ближе пополам.
					@pool_size = (@samples.size < @pool_size * 2) ? (@samples.size - 1) / 2 + 1 : @pool_size

					@assay_part = (0..@pool_size-1).to_a
					@left_part = (@pool_size..(@samples.size-1)).to_a

					@assay_samples = [@assay_part]
				else
					super
				end
			end
			init_attributes :@assay_part, :@left_part

			def infer
				results = super
				# Если в проверенной части не позитивов, а планировщику на входе сообщили, что позитивы
				# есть, и их число равно размеру оставшеся части, то все они считаются позитивами.
				# Чаще всего это бывает когда в оставшейся части находится только один сэмпл.
				if @min_positives > 0 && @min_positives == @left_part.size && @assay_output[0] == false
					@left_part.each do |sample|
						results[sample] = true
					end
				end
				results
			end

			def next_planners
				# Если первая группа тестов дала негативный результат, это значит, что в оставшейся
				# части точно есть позитив, и ее можно делить пополам.
				# Но если в первой группе есть позитив, то это не меняет вероятности для оставшейся
				# группы, поэтому первую группу делим пополам, а вторую проверяем без изменений.
				descendants = []


				if @min_positives > 0
					left_samples = @left_part.size > 0 ? @left_part.map{|i| @samples[i]} : nil
					if @assay_output[0] == false
						# Если это не единственная альтернатива, то нам еще нужно ее проверять
						if left_samples && left_samples.size != @min_positives
							reduced_limit = [@pool_size, (left_samples.size - 1) / SECTION_FACTOR + 1].min
							descendants << self.class.new(@strategy, left_samples, pool_size: reduced_limit, batch: @batch+[1], min_positives: 1)
						end
					else
						# В выполненной группе выявлен позитив, и мы сообщаем об этом планировщик-потомок по этой группе.
						# А в left_samples уверенности в наличии позитива нет.
						if @assay_part.size > 1 || @assay_output[0] == nil
							done_samples = @assay_part.map{|i| @samples[i]}
							reduced_limit = (@pool_size - 1) / SECTION_FACTOR + 1
							descendants << self.class.new(@strategy, done_samples, pool_size: reduced_limit, batch: @batch+[0], min_positives: 1)
						end

						if left_samples
							descendants << self.class.new(@strategy, left_samples, pool_size: @pool_size, batch: @batch+[1])
						end
					end
				else
					@assay_samples.each_with_index do |samples_indices, assay|
						next if @assay_output[assay] == false	# не проверяем группы с отрицательным тестом
						next if @assay_output[assay] == true && samples_indices.size == 1 # не проверяет группы из одного элемента с положительным тестом
						pool_size = (@pool_size - 1) / SECTION_FACTOR + 1
						samples = samples_indices.map{|i| @samples[i]}
						descendants << self.class.new(@strategy, samples, pool_size: pool_size, batch: @batch+[assay], min_positives: 1)
					end

				end

				descendants
			end

		end # class PoolBisectorWaiting
	end # module Planner
end # module LTO

=begin

NB: min_positives - в некоторых случаях может быть больше единицы, например, если мы объединим 3 набора сэмплов, каждый из которых имеет min_positives 1, то в итоговом наборе min_positives будет 3. При делении такого набора если одна часть оказалась в пуле false, то оставшаяся часть сохраняет min_positives 3. Но если первая часть окажется true, то обе части станут min_positives 1.

=end
