require 'ruby_helper'
module LTO
	module Strategy

		class Base

			# Создает один или несколько планировщиков следующего этапа
			# min_positives: nil, max_positives: nil
			def planners planner_class, samples, **kwargs
				raise NotImplementedError
			end

			include InstanceVariablesEqual

			def dump_init
				{}
			end

			def self.load_init hash
				new
			end
		end

		class Loop < Base

			# Создает планировщик такого же класса, как и вызвавший его планировщик
			def planners planner_class, samples, **kwargs
				[planner_class.new(self, samples, **kwargs)]
			end
		end # class Loop

		class PoolLoop < Loop

			attr_reader :pool_limit
			def initialize pool_limit
				@pool_limit = pool_limit
			end

			# Создает планировщик такого же класса, как и вызвавший его планировщик
			def planners planner_class, samples, **kwargs
				[planner_class.new(self, samples, **kwargs)]
			end

			def dump_init
				{ pool_limit: @pool_limit }
			end

			def self.load_init hash
				new hash[:pool_limit]
			end
		end # class Linear

		class PoolLinear < PoolLoop

			class << self
				attr_accessor :classes
				def new_class *classes
					Class.new(self) do
						self.classes = classes
					end
				end
			end

			attr_reader :pool_limit
			def initialize pool_limit
				super
				@depth = 0
			end

			def planners planner, samples, **kwargs
				# создаем копию текущей стратегии, меняем в ней атрибуты,
				# и созданем новые планировщики с новой копией
				dup.dive_planners planner, samples, **kwargs
			end

			def dive_planners planner, samples, **kwargs
				if @depth < self.class.classes.size 
					@klass = self.class.classes[@depth]
				end
				@depth += 1

				[@klass.new(self, samples, **kwargs)]
			end

			def dump_init
				ret = super
				ret[:depth] = @depth
				ret[:klass] = @klass.name if @klass
				ret
			end

			def self.load_init hash
				hash.transform_keys!(&:to_sym)
				obj = new hash[:pool_limit]
				obj.instance_variable_set(:@depth, hash[:depth]) if hash[:depth]
				obj.instance_variable_set(:@klass, Kernel.const_get(hash[:klass])) if hash[:klass]
				obj
			end

			require 'lto/planner/pool_bisector'
			require 'lto/planner/pool_bisector_waiting'
			require 'lto/planner/multipool_enumerator'
			require 'lto/planner/direct'

			DIRECT  = self.new_class LTO::Planner::Direct
			SIMPLE  = self.new_class LTO::Planner::PoolBisector, LTO::Planner::Direct
			OPTIMAL = self.new_class LTO::Planner::PoolBisector, LTO::Planner::MultipoolEnumerator, LTO::Planner::PoolBisectorWaiting

		end # class Linear

	end # module Strategy
end # module LTO

